function dynare_estimation(var_list,varargin)
% function dynare_estimation(var_list)
% runs the estimation of the model
%
% INPUTS
%   var_list:  selected endogenous variables vector
%
% OUTPUTS
%   none
%
% SPECIAL REQUIREMENTS
%   none

% Copyright (C) 2003-2008 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

global options_ oo_ M_ oo_recursive_

%% Decide if a DSGE or DSGE-VAR has to be estimated.
if ~isempty(strmatch('dsge_prior_weight',M_.param_names))
    options_.bvar_dsge = 1;
end

var_list = check_list_of_variables(options_, M_, var_list);
options_.varlist = var_list;

if isfield(options_,'nobs')
    nobs = options_.nobs;
else
    nobs = [];
end

nnobs = length(nobs);
horizon = options_.forecast;

if nnobs > 1
    if nargin > 1
        dname = vargin{1};
    else
        dname = M_.fname;
    end
    for i=1:nnobs
        options_.nobs = nobs(i);
        dynare_estimation_1(var_list,[dname '_' int2str(nobs(i))]);
        oo_recursive_{nobs(i)} = oo_;
    end
else
    dynare_estimation_1(var_list,varargin{:});
end

if nnobs > 1 & horizon > 0
    mh_replic = options_.mh_replic;
    rawdata = read_variables(options_.datafile,options_.varobs,[],options_.xls_sheet,options_.xls_range);
    gend = options_.nobs;
    rawdata = rawdata(options_.first_obs:options_.first_obs+gend-1,:);

    % Take the log of the variables if needed
    if options_.loglinear & ~options_.logdata   % and if the data are not in logs, then...
        rawdata = log(rawdata);
    end
    
    endo_names = M_.endo_names;
    n_varobs = size(options_.varobs,1);
    
    if isempty(var_list)
        var_list = endo_names;
        nvar    = size(endo_names,1);
        SelecVariables = transpose(1:nvar);
    else
        nvar = size(var_list,1);
        SelecVariables = [];
        for i=1:nvar
            if ~isempty(strmatch(var_list(i,:),endo_names,'exact'))
                SelecVariables = [SelecVariables;strmatch(var_list(i,:),endo_names, ...
                    'exact')];
            else
                error(['Estimation: ' var_list(i,:) ' isn''t an endogenous' ...
                    'variable'])
            end
        end
    end
  
   % nvar  #choice of variable list
   % n_varobs #obs variable list
   
    if n_varobs <= nvar
        IdObs    = zeros(n_varobs,1);
        for j=1:n_varobs
            for i=1:nvar
                iobs = strmatch(options_.varobs(j,:),var_list,'exact');
            end
            if ~isempty(iobs)
                IdObs(j,1) = iobs;
            end
        end
        
    else
        
        IdObs    = zeros(n_varobs,1);
        for j=1:nvar
            iobs = strmatch(var_list(j,:),options_.varobs,'exact');
            if ~isempty(iobs)
                IdObs(iobs,1) = j;
            end
        end
        
        
    end
    
    PredError = nan(nnobs,size(var_list,1),horizon);
    PredError_RW = nan(nnobs,size(var_list,1),horizon);
    Preds = nan(nnobs,size(var_list,1),horizon);
    DataFixedHorizon = nan(nnobs,size(var_list,1),horizon);
    
    k = 3+min(nobs(end)-nobs(1)+horizon, ...
        size(rawdata,1)-nobs(1));
    data2 = rawdata(end-k+1:end,:);
    [nbplt,nr,nc,lr,lc,nstar] = pltorg(nvar);
    m = 1;
    for i = 1:size(var_list,1)
        if mod(i,nstar) == 1
            hfig = figure('Name','Out of sample forecasts');
            m = 1;
        end
        subplot(nr,nc,m)
        hold on
        if any(i==IdObs)
            k2 = find(i==IdObs);
            if options_.loglinear == 1
                plot(1:k,exp(data2(end-k+1:end,k2))','-k','linewidth',2);
            else
                plot(1:k,data2(end-k+1:end,k2)','-k','linewidth',2);
            end
            offsetx = 3;
        else
            offsetx = 0;
        end
        vname = deblank(var_list(i,:));
        maxlag = M_.maximum_lag;
        for j=1:nnobs
            if mh_replic > 0
                eval(['oo_.RecursiveForecast.Mean.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.MeanForecast.Mean.' ...
                    vname '(maxlag+1:end);']);
                eval(['oo_.RecursiveForecast.HPDinf.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.MeanForecast.HPDinf.' ...
                    vname '(maxlag+1:end);']);
                eval(['oo_.RecursiveForecast.HPDsup.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.MeanForecast.HPDsup.' ...
                    vname '(maxlag+1:end);']);
                eval(['oo_.RecursiveForecast.HPDTotalinf.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.PointForecast.HPDinf.' ...
                    vname '(maxlag+1:end);']);
                eval(['oo_.RecursiveForecast.HPDTotalsup.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.PointForecast.HPDsup.' ...
                    vname '(maxlag+1:end);']);
            else
                eval(['oo_.RecursiveForecast.Mean.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.forecast.Mean.' ...
                    vname ';']);
                eval(['oo_.RecursiveForecast.HPDinf.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.forecast.HPDinf.' ...
                    vname ';']);
                eval(['oo_.RecursiveForecast.HPDsup.' vname '(j,:) =' ...
                    'oo_recursive_{' int2str(nobs(j))  '}.forecast.HPDsup.' ...
                    vname ';']);
            end
            x = offsetx+nobs(j)-nobs(1)+(1:horizon);
            y = eval(['oo_.RecursiveForecast.Mean.' vname '(j,:)']);
            y1 = eval(['oo_.RecursiveForecast.HPDinf.' vname '(j,:)']);
            y2 = eval(['oo_.RecursiveForecast.HPDsup.' vname '(j,:)']);
            plot(x,y,'-b','linewidth',2)
            
            if any(i==IdObs)   
                % ******************
                len=min(length(data2(x(1):end,k2)),length(y));  
                PredError(j,i,1:len) = y(1:len) - data2(x(1):x(1)+len-1,k2)'; % KW
                PredError_RW(j,i,1:len) = repmat(data2(x(1)-1,k2)',size(data2(x(1):x(1)+len-1,k2)'))-data2(x(1):x(1)+len-1,k2)'; 
                Preds(j,i,1:len) = y(1:len);  
                DataFixedHorizon(j,i,1:size(data2(x(1):end,k2)',2)) = data2(x(1):end,k2)';
            end           
            
            if mh_replic
                y3 = eval(['oo_.RecursiveForecast.HPDTotalinf.' vname '(j,:)']);
                y4 = eval(['oo_.RecursiveForecast.HPDTotalsup.' vname ...
                    '(j,:)']);
                plot(x,y3,'--r', ...
                    'linewidth',1.5)
                plot(x,y4,'--r','linewidth',1.5)
            end
        end

        if any(k==IdObs)
            plot([offsetx+1 offsetx+1],ylim,'-c')
        end
        box on
        title(vname,'Interpreter','none')
        
        if mod(i,nstar) == 0 || i == size(var_list,1)
            eval(['print -depsc2 ' M_.fname '_RollingForecast' num2str(hfig) '_.eps']);
            saveas(hfig,[M_.fname '_RollingForecast' num2str(hfig) '.fig']);
        end
        
        hold off
        m = m + 1;
    end
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%% Accuracy of point forecasts %%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    MPE=nan(horizon,size(var_list,1));
    RMSE=nan(horizon,size(var_list,1));
    RMSE_RW=nan(horizon,size(var_list,1));
    MAE=nan(horizon,size(var_list,1));
    MAPE=nan(horizon,size(var_list,1));
    LogDetMSSE=zeros(horizon,size(var_list,1));
    TraceMSSE=zeros(horizon,size(var_list,1));
    
    for j=1:horizon
        AvailableDataObs = find(isnan(PredError(:,1,j))==0);
        if isempty(AvailableDataObs)==0
            MPE(j,:)=100*mean(PredError(AvailableDataObs,:,j)./DataFixedHorizon(AvailableDataObs,:,j),1);
            RMSE(j,:)=sqrt(mean(PredError(AvailableDataObs,:,j).^2,1));
            RMSE_RW(j,:)=sqrt(mean(PredError_RW(AvailableDataObs,:,j).^2,1));
            MAE(j,:)=mean(abs(PredError(AvailableDataObs,:,j)),1);
            MAPE(j,:)=100*mean(abs(PredError(AvailableDataObs,:,j)./DataFixedHorizon(AvailableDataObs,:,j)),1);
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       Printing routine for univariate forecasting measures        %%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    OutFileName = M_.fname;
    fid=fopen(strcat('RMSE_',OutFileName,'.txt'),'wt'); % Open file to write the results to
    
    AllUniMeasures(:,:,:,1)=RMSE;
    AllUniMeasures(:,:,:,2)=MAE;
    AllUniMeasures(:,:,:,3)=MAPE;
    AllUniMeasures(:,:,:,4)=MPE;
    AllUniMeasures(:,:,:,5)=RMSE_RW;
    AllMultiMeasures{1}=LogDetMSSE;
    AllMultiMeasures{2}=TraceMSSE;
    
    MeasureLabels{1}='RMSE';
    MeasureLabels{2}='MAE';
    MeasureLabels{3}='MAPE';
    MeasureLabels{4}='MPE';
    MeasureLabels{5}='RMSE_RW';
    ModelLabels = {M_.fname};
    PrintMeasures = 1:5;
    PrintModels = 1;
    PrintVars = 1:size(var_list,1);
    VarLabels = var_list;
    NPrintHorizons=horizon;
    EvalHorizons = [1:horizon];
    
    fprintf(fid,'\n');
    fprintf(fid,'--------------------------------------------------------------------------------------------------------------------\n');
    fprintf(fid,'                                      Measures of Point Forecast Accuracy                                           \n');
    fprintf(fid,'--------------------------------------------------------------------------------------------------------------------\n');
    
    %%%%%%%%%%%%%%%%%% Univariate measures %%%%%%%%%%%%%%%%%%%%
    for Measure=PrintMeasures
        fprintf(fid,'\n\n\n');
        fprintf(fid,'---------------------------------------------------------------------\n');
        fprintf(fid,['                         ',MeasureLabels{Measure},'                 \n']);
        fprintf(fid,'---------------------------------------------------------------------\n');
        
        for Model=PrintModels
            fprintf(fid,'\n');
            fprintf(fid,ModelLabels{Model});
            fprintf(fid,'\n');
            
            % Printing variable labels on top of each column
            fprintf(fid,'%7.7s','Horizon');
            for Var=PrintVars
                fprintf(fid,'%10.10s',VarLabels(Var,:));
            end
            fprintf(fid,'\n');
            
            % Printing Univariate forecast accuracy measures
            for h=1:NPrintHorizons
                fprintf(fid,'%7s',strcat(int2str(EvalHorizons(h)),'Q'));
                fprintf(fid,'%10.3f',AllUniMeasures(h,PrintVars,Model,Measure));
                fprintf(fid,'\n');
            end
        end
    end
        
    fclose(fid);
    winopen(strcat('RMSE_',OutFileName,'.txt'))
      
end